package com.pfc.app.nbff;

import java.util.List;

import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.Point;
import android.graphics.RectF;
import android.graphics.Paint.Style;
import android.net.Uri;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.RelativeLayout;
import android.widget.TextView;
import android.widget.Toast;

import com.google.android.maps.GeoPoint;
import com.google.android.maps.MapView;
import com.google.android.maps.Overlay;

public class AmigoOverlay extends Overlay {

	Context contextoauxiliar;
	ProgressDialog progresoCargaAmigos = null;
	/**
	* Variable que guarda la imagen a guardar para un amigo.
	*/
	private Bitmap mAmigo;
	
	/**
	* Variable que guarda la imagen a guardar para un amigo.
	*/
	private Bitmap mAmiga;
	
	/**
	* Variable que guarda la posición del amigo en el mapa
	*/
	private GeoPoint mGeoPunto;
	
	/**
	* Variable que guarda el id del amigo a mostrar en una etiqueta
	*/
	private String idUsuario;
	
	/**
	* Variable que guarda la etiqueta con el nick del amigo
	*/
	private String mEtiqueta;
	
	/** Método constructor de la clase.
	*
	* @param mapa Mapa donde se mostrará el overlay
	* @param geoPunt Punto del mapa donde está el amigo
	* @param texto Nombre del amigo
	*
	* @return Un elemento del tipo Overlay
	*/
	public AmigoOverlay(MapView mapa, GeoPoint geoPunto, String texto, String idUs) {
		
		//Copiamos los parámetros de entrada a las variables de la clase
		mAmigo = BitmapFactory.decodeResource(mapa.getResources(), R.drawable.friend_icon);
		mAmiga = BitmapFactory.decodeResource(mapa.getResources(), R.drawable.friend_icon);
		mGeoPunto = geoPunto;
		mEtiqueta = texto;
		idUsuario = idUs;
	}
	
	/** Método que devuelve el lugar donde está el amigo.
	*
	* @return Un GeoPoint con la latitud y longitud del amigo.
	*/	
	public GeoPoint getGeoPunto(){
	
		return mGeoPunto;
	}
	
	/** Método que devuelve el nombre del amigo.
	*
	* @return Nombre del amigo.
	*/
	public String getIdAmigo(){
		
		return idUsuario;
	}
	
	/**Método que se encarga de dibujar la imagen que se mostrará en el lugar donde se encuentra el Overlay.
	*
	* @param canvas El lienzo donde vamos a dibujar.
	* @param mapa El mapa donde se va a dibujar.
	* @param shadow Define si queremos dibujar una sombra o no.
	*/
	@Override
	public void draw(Canvas canvas, MapView mapa, boolean shadow) {
		
		//Vamos a dibujar por separado el icono del amigo y la etiqueta que contendrá su nombre
		drawAmigo(canvas, mapa, shadow);
		drawNombre(canvas, mapa, shadow);
	}
	
	/**Método que dibuja al icono que representa al amigo
	*
	* @param canvas El lienzo donde vamos a dibujar.
	* @param mapa El mapa donde se va a dibujar.
	* @param shadow Define si queremos dibujar una sombra o no.
	*/
	private void drawAmigo(Canvas canvas, MapView mapa, boolean shadow){
		//Declaramos un punto y pasamos de coordenadas a p�xeles de pantalla
		Point puntoOverlay = new Point();
		mapa.getProjection().toPixels(mGeoPunto,puntoOverlay);
		
		//Si no se dibuja la sombra
		if (!shadow) {
			//Dibujamos en el punto calculado el Bitmap de la imagen del amigo.
			canvas.drawBitmap(mAmigo, puntoOverlay.x - mAmigo.getWidth()/2, puntoOverlay.y - mAmigo.getHeight(),null);
		}
	}
	
	/** Método que dibuja la etiqueta con el nombre del amigo.
	* 
	* @param canvas El lienzo donde vamos a dibujar.
	* @param mapa El mapa donde se va a dibujar.
	* @param shadow Define si queremos dibujar una sombra o no.
	*/
	private void drawNombre(Canvas canvas, MapView mapa, boolean shadow){
		
		//Si no se dibuja la sombra.
		if (!shadow) {
			//Primero determinamos la relacion pixel-pantalla con las coordenadas.
			Point puntoOverlay = new Point();
			mapa.getProjection().toPixels(mGeoPunto,puntoOverlay);
			
			//Configuramos la ventana que mostrara el texto.
			int windowW = 25;
			int windowH = 25;
			//A más largo el texto, más longitud de etiqueta necesitaremos
			for(int i=0;i<mEtiqueta.length();i++)
				windowW+=5;
			//Vamos a crear el rectángulo donde irá el nombre del amigo
			RectF infoWindowRect = new RectF(0,0,windowW,windowH);
			int winOffsetX = puntoOverlay.x-windowW/2;
			int winOffsetY = puntoOverlay.y-windowH-mAmigo.getHeight();
			infoWindowRect.offset(winOffsetX, winOffsetY);
			
			//Dibujamos el interior de la ventana.
			canvas.drawRoundRect(infoWindowRect, 5, 5, getPinturaDentro());
			//Dibujamos el borde de la ventana.
			canvas.drawRoundRect(infoWindowRect, 5, 5, getPinturaBorde());
			
			//Dibujamos el nombre del amigo.
			int txtOffX = 10;
			int txtOffY = 15;
			//Dibujamos el nombre de amigo dentro del rectángulo
			canvas.drawText(mEtiqueta, winOffsetX+txtOffX, winOffsetY+txtOffY, getPinturaTexto());
		}
	}
	
	/**Método que devuelve el interior de un rectángulo
	*
	* @return El relleno del rectángulo
	*/
	public Paint getPinturaDentro() {
		
		Paint pinturaDentro = new Paint();
		pinturaDentro.setARGB(225, 75, 75, 75); //gray
		pinturaDentro.setAntiAlias(true);
			
		return pinturaDentro;
	}
	
	/** Método que devuelve el borde del rectángulo
	*
	* @return El borde del rectángulo
	*/
	public Paint getPinturaBorde() {
		
		Paint pinturaBorde = new Paint();
		pinturaBorde.setARGB(255, 255, 255, 255);
		pinturaBorde.setAntiAlias(true);
		pinturaBorde.setStyle(Style.STROKE);
		pinturaBorde.setStrokeWidth(2);
			
		return pinturaBorde;
	}
	
	/** Método que devuelve el estilo de la letra para el texto
	*
	* @return El estilo del texto
	*/
	public Paint getPinturaTexto() {
		
		Paint pinturaTexto = new Paint();
		pinturaTexto.setARGB(255, 255, 255, 255);
		pinturaTexto.setAntiAlias(true);
			
		return pinturaTexto;
	}
	
	/**Método obligatorio de implementar en la clase. Implementa el evento de pulsación sobre el Overlay del mapa.
	*
	* @param point Punto pulsado.
	* @param mapa El mapa al que pertenece el Overlay.
	*/
	@Override
	public boolean onTap(GeoPoint point, final MapView mapa) {
		
		 		
		 contextoauxiliar = mapa.getContext(); 
		 final List<Overlay> overlays  = mapa.getOverlays();
		 //boolean encontrado = false;
		 int index = 0;
		 //Creamos el punto pulsado y la zona a pulsar. Además le asignamos al punto pulsado
		 //la proyeccion del Geopunto que se pulsó en la pantalla.
		 RectF zonaParaPulsar;
		 Point puntoPulsado = new Point();
		 mapa.getProjection().toPixels(point, puntoPulsado);
		 Point puntoDeOverlay = new Point();
		 
		 boolean encontrado = false;
		 
		 Log.i("LogOnTapAmigo", "Entrando en el bucle... encontrado? " + encontrado);
		 //mientras no encontremos el punto y haya puntos que comprobar
		 Log.i("LogOnTapAmigo", "El número de overlays es..." + overlays.size());
		 while(!encontrado && index < overlays.size()){
			 //Definimos el tamaño del rectangulo de la chincheta.
			 zonaParaPulsar = new RectF();
			 zonaParaPulsar.set(-mAmigo.getWidth()/2,-mAmigo.getHeight(),mAmigo.getWidth()/2,0);
			 Log.i("LogOnTapAmigo", "Creada zona para pulsar..." + index);
			 try
			 {
				 //Recogemos uno de los overlays dentro del mapa.
				 final AmigoOverlay overlay = (AmigoOverlay) overlays.get(index);
				 Log.i("LogOnTapAmigo", "overlays.get");
				 mapa.getProjection().toPixels(overlay.getGeoPunto(), puntoDeOverlay);
				 Log.i("LogOnTapAmigo", "mapa.getProjection.toPixels");
				 //Definimos el offset del rectangulo dentro de la pantalla.
				 zonaParaPulsar.offset(puntoDeOverlay.x+1,puntoDeOverlay.y+1);
				 Log.i("LogOnTapAmigo", "Añadido offset");
				 //Si la zona pulsada contiene el punto
				 
				 if(zonaParaPulsar.contains(puntoPulsado.x,puntoPulsado.y)){
 
					 AlertDialog.Builder constructor = new AlertDialog.Builder(contextoauxiliar);
					 Log.i("LogOnTapAmigo", "creamos alertdialog.builder");
					 constructor.setCancelable(true);

					 Log.i("LogOnTapAmigo", "creamos alertdialog");
					 LayoutInflater inflater = LayoutInflater.from(contextoauxiliar);
					 View v = inflater.inflate(R.layout.interfaz_dialogo_perfil_usuario, null);

					 constructor.setView(v);
					
					 AlertDialog dialogo = constructor.create();
					 dialogo.setCanceledOnTouchOutside(true);

					 dialogo.show();
					 Log.i("LogOnTapAmigo", "Mostramos");
					 
					 
					 final TextView textNick = (TextView) dialogo.findViewById(R.id.TextViewNickPerfil);
					 final TextView textSaludo = (TextView) dialogo.findViewById(R.id.TextViewSaludoPerfil);
					 final TextView textDescripcion = (TextView) dialogo.findViewById(R.id.TextViewDescripcionPerfil);
					 final TextView textEmail = (TextView) dialogo.findViewById(R.id.TextViewEmailPerfil);
					 final TextView textMas = (TextView) dialogo.findViewById(R.id.TextViewVerMas);
					 final RelativeLayout layParteBaja = (RelativeLayout) dialogo.findViewById(R.id.RelativeLayoutParteBaja);
					 
					 final TextView textNombre = (TextView) dialogo.findViewById(R.id.TextViewNombrePerfil);
					 final TextView textApellidos = (TextView) dialogo.findViewById(R.id.TextViewApellidosPerfil);
					 final TextView textSexo = (TextView) dialogo.findViewById(R.id.TextViewSexoPerfil);
					 final TextView textAltura = (TextView) dialogo.findViewById(R.id.TextViewAlturaPerfil);
					 final TextView textPeso = (TextView) dialogo.findViewById(R.id.TextViewPesoPerfil);
					 final TextView textComplexion = (TextView) dialogo.findViewById(R.id.TextViewComplexionPerfil);
					 final TextView textEdad = (TextView) dialogo.findViewById(R.id.TextViewEdadPerfil);
					 final TextView textEstudios = (TextView) dialogo.findViewById(R.id.TextViewEstudiosPerfil);
					 
					 final ImageView escribir = (ImageView) dialogo.findViewById(R.id.ImageButtonEscribir);
					 final ImageView añadirAmigo = (ImageView) dialogo.findViewById(R.id.ImageButtonAnadir);
					 final ImageView galeria = (ImageView) dialogo.findViewById(R.id.ImageButtonGaleria);
					 
					 String perfil = ActividadMapa.devuelvePerfil(ActividadMapa.conectarPerfil(ActividadMapa.cntxt,overlay.idUsuario));
					 String nick = ActividadMapa.desencriptaNickPerfil(perfil);
					 String emilio = ActividadMapa.desencriptaEmailPerfil(perfil);
					 String saludo = ActividadMapa.desencriptaSaludoPerfil(perfil);
					 String descript = ActividadMapa.desencriptaDescripcionPerfil(perfil);
					 
					 String nom = ActividadMapa.desencriptaNombrePerfil(perfil);
					 String apell = ActividadMapa.desencriptaApellidosPerfil(perfil);
					 String sexo = ActividadMapa.desencriptaSexoPerfil(perfil);
					 String altur = ActividadMapa.desencriptaAlturaPerfil(perfil);
					 String peso = ActividadMapa.desencriptaPesoPerfil(perfil);
					 String compl = ActividadMapa.desencriptaComplexionPerfil(perfil);
					 String edad = ActividadMapa.desencriptaEdadPerfil(perfil);
					 String estud = ActividadMapa.desencriptaEstudiosPerfil(perfil);
					 
					 textNick.setText(nick);
					 textSaludo.setText(saludo);
					 textDescripcion.setText(descript);
					 textEmail.setText(emilio);
					 
					 textNombre.setText(nom);
					 textApellidos.setText(apell);
					 textSexo.setText(sexo);
					 textAltura.setText(altur);
					 textPeso.setText(peso);
					 textComplexion.setText(compl);
					 textEdad.setText(edad);
					 textEstudios.setText(estud);
					 
					 if (ActividadMapa.esAmigo(ActividadMapa.idUsuario,idUsuario).equals("false")){
						 añadirAmigo.setVisibility(View.VISIBLE);
					 }
					 else{
						 galeria.setVisibility(View.VISIBLE);
					 }
					 
					 
					 
					 textMas.setOnClickListener(new OnClickListener() {
						
						public void onClick(View v) {
							layParteBaja.setVisibility(View.VISIBLE);
							textMas.setVisibility(View.GONE);
							
						}
					});
					 
					 escribir.setOnClickListener(new OnClickListener() {
						
						public void onClick(View v) {
							
							AlertDialog.Builder constructorMensaje = new AlertDialog.Builder(contextoauxiliar);
							Log.i("escribirMensaje", "creamos alertdialog.builder");
							constructorMensaje.setCancelable(true);
							constructorMensaje.setTitle(R.string.tituloMensaje);
							
							final EditText mensaje = new EditText(contextoauxiliar);

							Log.i("escribirMensaje", "creamos editText");
							

							constructorMensaje.setView(mensaje);
							
							constructorMensaje.setPositiveButton(R.string.enviaMensaje, new DialogInterface.OnClickListener() {
								
								public void onClick(DialogInterface dialog, int which) {
									ActividadMapa.enviaMensaje(idUsuario,mensaje.getText().toString());
									Toast.makeText(contextoauxiliar, R.string.mensajeExito, Toast.LENGTH_LONG).show();
									
								}
							});
							constructorMensaje.create().show();
							
						}
					});
					 
					 añadirAmigo.setOnClickListener(new OnClickListener() {
						
						public void onClick(View v) {
							
							//if (ActividadMapa.enviaPeticionAmigoId(ActividadMapa.idUsuario,idUsuario).equals("true")){
								ActividadMapa.enviaPeticionAmigoId(ActividadMapa.idUsuario,idUsuario);
								Toast.makeText(contextoauxiliar, R.string.peticionAmigoExito, Toast.LENGTH_LONG).show();
								añadirAmigo.setVisibility(View.GONE);
							//}else
								//Toast.makeText(contextoauxiliar, R.string.peticionAmigoNoExito, Toast.LENGTH_LONG).show();
							
							
						}
					});
					 
					 galeria.setOnClickListener(new OnClickListener() {
						
						public void onClick(View v) {
							
							Intent intent1 = new Intent("android.intent.action.VIEW", Uri.parse("http://aperezba.dyndns.org:9080/webService/PruebaAction.action?parametroJSON="+idUsuario));
							contextoauxiliar.startActivity(intent1);
							
						}

						
					});
					 
					 Log.i("LogOnTapAmigo", "PRESaliendo del bucle... encontrado? " + encontrado);
					 encontrado = true;
					 Log.i("LogOnTapAmigo", "Saliendo del bucle... encontrado?" + encontrado);
					 return true;
				 }	
			 }
			 catch(Exception exc)
			 {
				 Log.i("LogOnTapAmigoExcepcion", "Fallo en el onTap: " + exc);
			 }
			 index++;
		}
		return false;
	}

	/* Métodos que ayudan a hacer tareas en background mientras en pantalla se ve el progressDialog
	class FinishThread extends Thread{
		
		public void run(){
				
			//Creamos el constructor del diálogo
			 
			 
			 
			 
			progresoCargaAmigos.dismiss();
		}
				
	}   
	
     class LoadingThread extends Thread{
			
			Handler manejador;
			public LoadingThread(Handler handler){
				super();
				this.manejador=handler;
			}
			public void run(){
				try{
					
				}catch (Exception e){
					
				}
				manejador.post(new FinishThread());
			}
		}
     */
	
	
}
